@@ -6,11 +6,11 @@ Test::WWW::Mechanize - Testing-specific WWW::Mechanize subclass
 
 =head1 VERSION
 
-Version 1.14
+Version 1.18
 
 =cut
 
-our $VERSION = '1.14';
+our $VERSION = '1.18';
 
 =head1 SYNOPSIS
 
@@ -42,6 +42,23 @@ This is equivalent to:
 
 but has nicer diagnostics if they fail.
 
+Default descriptions will be supplied for most methods if you omit them. e.g.
+
+    my $mech = Test::WWW::Mechanize->new;
+    $mech->get_ok( 'http://petdance.com/' );
+    $mech->base_is( 'http://petdance.com/' );
+    $mech->title_is( "Invoice Status" );
+    $mech->content_contains( "Andy Lester" );
+    $mech->content_like( qr/(cpan|perl)\.org/ );
+
+results in
+
+    ok - Got 'http://petdance.com/' ok
+    ok - Base is 'http://petdance.com/'
+    ok - Title is 'Invoice Status'
+    ok - Content contains 'Andy Lester'
+    ok - Content is like '(?-xism:(cpan|perl)\.org)'
+
 =cut
 
 use warnings;
@@ -50,6 +67,7 @@ use strict;
 use WWW::Mechanize ();
 use Test::LongString;
 use Test::Builder ();
+use Carp ();
 use Carp::Assert::More;
 
 use base 'WWW::Mechanize';
@@ -75,7 +93,7 @@ sub new {
     return $self;
 }
 
-=head1 METHODS
+=head1 METHODS: GETTING & POSTING
 
 =head2 $mech->get_ok($url, [ \%LWP_options ,] $desc)
 
@@ -84,6 +102,8 @@ the second argument needs to be a hash reference, not a hash. Like
 well-behaved C<*_ok()> functions, it returns true if the test passed,
 or false if not.
 
+A default description of "GET $url" is used if none if provided.
+
 =cut
 
 sub get_ok {
@@ -115,6 +135,10 @@ sub get_ok {
     $self->get( $url, %opts );
     my $ok = $self->success;
 
+    if ( not defined $desc ) {
+        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
+        $desc = "GET $url";
+    }
     $Test->ok( $ok, $desc );
     if ( !$ok ) {
         $Test->diag( $self->status );
@@ -124,6 +148,222 @@ sub get_ok {
     return $ok;
 }
 
+=head2 $mech->post_ok( $url, [ \%LWP_options ,] $desc )
+
+A wrapper around WWW::Mechanize's post(), with similar options, except
+the second argument needs to be a hash reference, not a hash. Like
+well-behaved C<*_ok()> functions, it returns true if the test passed,
+or false if not.
+
+A default description of "POST to $url" is used if none if provided.
+
+=cut
+
+sub post_ok {
+    my $self = shift;
+    my $url = shift;
+
+    my $desc;
+    my %opts;
+
+    if ( @_ ) {
+        my $flex = shift; # The flexible argument
+
+        if ( !defined( $flex ) ) {
+            $desc = shift;
+        }
+        elsif ( ref $flex eq 'HASH' ) {
+            %opts = %$flex;
+            $desc = shift;
+        }
+        elsif ( ref $flex eq 'ARRAY' ) {
+            %opts = @$flex;
+            $desc = shift;
+        }
+        else {
+            $desc = $flex;
+        }
+    } # parms left
+
+    if ( not defined $desc ) {
+        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
+        $desc = "POST $url";
+    }
+    $self->post( $url, \%opts );
+    my $ok = $self->success;
+    $Test->ok( $ok, $desc );
+    if ( !$ok ) {
+        $Test->diag( $self->status );
+        $Test->diag( $self->response->message ) if $self->response;
+    }
+
+    return $ok;
+}
+
+
+=head2 submit_form_ok( \%parms [, $desc] )
+
+Makes a C<submit_form()> call and executes tests on the results.
+The form must be found, and then submitted successfully.  Otherwise,
+this test fails.
+
+I<%parms> is a hashref containing the parms to pass to C<submit_form()>.
+Note that the parms to C<submit_form()> are a hash whereas the parms to
+this function are a hashref.  You have to call this function like:
+
+    $agent->submit_form_ok( {n=>3}, "looking for 3rd link" );
+
+As with other test functions, C<$desc> is optional.  If it is supplied
+then it will display when running the test harness in verbose mode.
+
+Returns true value if the specified link was found and followed
+successfully.  The L<HTTP::Response> object returned by submit_form()
+is not available.
+
+=cut
+
+sub submit_form_ok {
+    my $self = shift;
+    my $parms = shift || {};
+    my $desc = shift;
+
+    if ( ref $parms ne 'HASH' ) {
+       Carp::croak "FATAL: parameters must be given as a hashref";
+    }
+
+    # return from submit_form() is an HTTP::Response or undef
+    my $response = $self->submit_form( %$parms );
+
+    my $ok;
+    my $error;
+    if ( !$response ) {
+        $error = "No matching form found";
+    }
+    else {
+        if ( $response->is_success ) {
+            $ok = 1;
+        }
+        else {
+            $error = $response->as_string;
+        }
+    }
+
+    $Test->ok( $ok, $desc );
+    $Test->diag( $error ) if $error;
+
+    return $ok;
+}
+
+
+=head2 $mech->follow_link_ok( \%parms [, $desc] )
+
+Makes a C<follow_link()> call and executes tests on the results.
+The link must be found, and then followed successfully.  Otherwise,
+this test fails.
+
+I<%parms> is a hashref containing the parms to pass to C<follow_link()>.
+Note that the parms to C<follow_link()> are a hash whereas the parms to
+this function are a hashref.  You have to call this function like:
+
+    $mech->follow_link_ok( {n=>3}, "looking for 3rd link" );
+
+As with other test functions, C<$desc> is optional.  If it is supplied
+then it will display when running the test harness in verbose mode.
+
+Returns a true value if the specified link was found and followed
+successfully.  The L<HTTP::Response> object returned by follow_link()
+is not available.
+
+=cut
+
+sub follow_link_ok {
+    my $self = shift;
+    my $parms = shift || {};
+    my $desc = shift;
+
+    if (!defined($desc)) {
+        my $parms_str = join(", ", map { join("=", $_, $parms->{$_}) } keys(%$parms));
+        $desc = "Followed link with '$parms_str'" if !defined($desc);
+    }
+
+    if ( ref $parms ne 'HASH' ) {
+       Carp::croak "FATAL: parameters must be given as a hashref";
+    }
+
+    # return from follow_link() is an HTTP::Response or undef
+    my $response = $self->follow_link( %$parms );
+
+    my $ok;
+    my $error;
+    if ( !$response ) {
+        $error = "No matching link found";
+    }
+    else {
+        if ( $response->is_success ) {
+            $ok = 1;
+        }
+        else {
+            $error = $response->as_string;
+        }
+    }
+
+    $Test->ok( $ok, $desc );
+    $Test->diag( $error ) if $error;
+
+    return $ok;
+}
+
+=head1 METHODS: CONTENT CHECKING
+
+=head2 $mech->html_lint_ok( [$msg] )
+
+Checks the validity of the HTML on the current page.  If the page is not
+HTML, then it fails.  The URI is automatically appended to the I<$msg>.
+
+Note that HTML::Lint must be installed for this to work.  Otherwise,
+it will blow up.
+
+=cut
+
+sub html_lint_ok {
+    my $self = shift;
+    my $msg = shift;
+
+    eval 'require HTML::Lint';
+    $@ and die 'html_lint_ok cannot run without HTML::Lint';
+
+    my $uri = $self->uri;
+    $msg = $msg ? "$msg ($uri)" : $uri;
+
+    my $ok;
+
+    if ( $self->is_html ) {
+
+        my $lint = HTML::Lint->new;
+        $lint->parse( $self->content );
+
+        my @errors = $lint->errors;
+        my $nerrors = @errors;
+        if ( $nerrors ) {
+            $ok = $Test->ok( 0, $msg );
+            $Test->diag( "HTML::Lint errors for $uri" );
+            $Test->diag( $_->as_string ) for @errors;
+            my $s = $nerrors == 1 ? '' : 's';
+            $Test->diag( "$nerrors error$s on the page" );
+        }
+        else {
+            $ok = $Test->ok( 1, $msg );
+        }
+    }
+    else {
+        $ok = $Test->ok( 0, $msg );
+        $Test->diag( q{This page doesn't appear to be HTML, or didn't get the proper text/html content type returned.} );
+    }
+
+    return $ok;
+}
+
+
 =head2 $mech->title_is( $str [, $desc ] )
 
 Tells if the title of the page is the given string.
@@ -136,6 +376,7 @@ sub title_is {
     my $self = shift;
     my $str = shift;
     my $desc = shift;
+    $desc = "Title is '$str'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return is_string( $self->title, $str, $desc );
@@ -153,6 +394,7 @@ sub title_like {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Title is like '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return like_string( $self->title, $regex, $desc );
@@ -170,6 +412,7 @@ sub title_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Title unlike '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unlike_string( $self->title, $regex, $desc );
@@ -187,6 +430,7 @@ sub base_is {
     my $self = shift;
     my $str = shift;
     my $desc = shift;
+    $desc = "Base is '$str'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return is_string( $self->base, $str, $desc );
@@ -204,6 +448,7 @@ sub base_like {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Base is like '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return like_string( $self->base, $regex, $desc );
@@ -221,6 +466,7 @@ sub base_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Base unlike '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unlike_string( $self->base, $regex, $desc );
@@ -238,6 +484,8 @@ sub content_is {
     my $desc = shift;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
+    $desc = qq{Content is "$str"} if !defined($desc);
+
     return is_string( $self->content, $str, $desc );
 }
 
@@ -253,6 +501,11 @@ sub content_contains {
     my $desc = shift;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
+    if ( ref($str) eq 'REGEX' ) {
+        diag( "content_contains takes a string, not a regex" );
+    }
+    $desc = qq{Content contains "$str"} if !defined($desc);
+
     return contains_string( $self->content, $str, $desc );
 }
 
@@ -268,6 +521,11 @@ sub content_lacks {
     my $desc = shift;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
+    if ( ref($str) eq 'REGEX' ) {
+        diag( 'content_lacks takes a string, not a regex' );
+    }
+    $desc = qq{Content lacks "$str"} if !defined($desc);
+
     return lacks_string( $self->content, $str, $desc );
 }
 
@@ -281,6 +539,7 @@ sub content_like {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Content is like '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return like_string( $self->content, $regex, $desc );
@@ -296,6 +555,7 @@ sub content_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Content unlike '$regex'" if !defined($desc);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unlike_string( $self->content, $regex, $desc );
@@ -312,6 +572,7 @@ sub has_tag {
     my $tag  = shift;
     my $text = shift;
     my $desc = shift;
+    $desc = "Page has $tag tag with '$text'" if !defined($desc);
 
     my $found = $self->_tag_walk( $tag, sub { $text eq $_[0] } );
 
@@ -330,6 +591,7 @@ sub has_tag_like {
     my $tag  = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "Page has $tag tag like '$regex'" if !defined($desc);
 
     my $found = $self->_tag_walk( $tag, sub { $_[0] =~ $regex } );
 
@@ -375,6 +637,7 @@ Follow all links on the current page and test for HTTP status 200
 sub page_links_ok {
     my $self = shift;
     my $desc = shift;
+    $desc = "All links ok" if !defined($desc);
 
     my @links = $self->followable_links();
     my @urls = _format_links(\@links);
@@ -388,7 +651,7 @@ sub page_links_ok {
     return $ok;
 }
 
-=head2 $mech->page_links_content_like( $regex,[ $desc ] )
+=head2 $mech->page_links_content_like( $regex [, $desc ] )
 
 Follow all links on the current page and test their contents for I<$regex>.
 
@@ -401,6 +664,7 @@ sub page_links_content_like {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "All links are like '$regex'" if !defined($desc);
 
     my $usable_regex=$Test->maybe_regex( $regex );
     unless(defined( $usable_regex )) {
@@ -421,7 +685,7 @@ sub page_links_content_like {
     return $ok;
 }
 
-=head2 $mech->page_links_content_unlike( $regex,[ $desc ] )
+=head2 $mech->page_links_content_unlike( $regex [, $desc ] )
 
 Follow all links on the current page and test their contents do not
 contain the specified regex.
@@ -435,6 +699,7 @@ sub page_links_content_unlike {
     my $self = shift;
     my $regex = shift;
     my $desc = shift;
+    $desc = "All links are unlike '$regex'" if !defined($desc);
 
     my $usable_regex=$Test->maybe_regex( $regex );
     unless(defined( $usable_regex )) {
@@ -478,6 +743,7 @@ sub links_ok {
     my $desc = shift;
 
     my @urls = _format_links( $links );
+    $desc = _default_links_desc(\@urls, "are ok") if !defined($desc);
     my @failures = $self->_check_links_status( \@urls );
     my $ok = (@failures == 0);
 
@@ -507,6 +773,7 @@ sub link_status_is {
     my $desc = shift;
 
     my @urls = _format_links( $links );
+    $desc = _default_links_desc(\@urls, "have status $status") if !defined($desc);
     my @failures = $self->_check_links_status( \@urls, $status );
     my $ok = (@failures == 0);
 
@@ -536,6 +803,7 @@ sub link_status_isnt {
     my $desc = shift;
 
     my @urls = _format_links( $links );
+    $desc = _default_links_desc(\@urls, "do not have status $status") if !defined($desc);
     my @failures = $self->_check_links_status( \@urls, $status, 'isnt' );
     my $ok = (@failures == 0);
 
@@ -573,6 +841,7 @@ sub link_content_like {
     }
 
     my @urls = _format_links( $links );
+    $desc = _default_links_desc(\@urls, "are like '$regex'") if !defined($desc);
     my @failures = $self->_check_links_content( \@urls, $regex );
     my $ok = (@failures == 0);
 
@@ -609,6 +878,7 @@ sub link_content_unlike {
     }
 
     my @urls = _format_links( $links );
+    $desc = _default_links_desc(\@urls, "are not like '$regex'") if !defined($desc);
     my @failures = $self->_check_links_content( \@urls, $regex, 'unlike' );
     my $ok = (@failures == 0);
 
@@ -618,6 +888,13 @@ sub link_content_unlike {
     return $ok;
 }
 
+# Create a default description for the link_* methods, including the link count.
+sub _default_links_desc {
+    my ($urls, $desc_suffix) = @_;
+    my $url_count = scalar(@$urls);
+    return sprintf("%d link%s %s", $url_count, $url_count == 1 ? "" : "s", $desc_suffix);
+}
+
 # This actually performs the status check of each url.
 sub _check_links_status {
     my $self = shift;
@@ -701,55 +978,6 @@ sub _format_links {
     return @urls;
 }
 
-=head2 $mech->follow_link_ok( \%parms [, $comment] )
-
-Makes a C<follow_link()> call and executes tests on the results.
-The link must be found, and then followed successfully.  Otherwise,
-this test fails.
-
-I<%parms> is a hashref containing the parms to pass to C<follow_link()>.
-Note that the parms to C<follow_link()> are a hash whereas the parms to
-this function are a hashref.  You have to call this function like:
-
-    $mech->follow_link_ok( {n=>3}, "looking for 3rd link" );
-
-As with other test functions, C<$comment> is optional.  If it is supplied
-then it will display when running the test harness in verbose mode.
-
-Returns true value if the specified link was found and followed
-successfully.  The HTTP::Response object returned by follow_link()
-is not available.
-
-=cut
-
-sub follow_link_ok {
-    my $self = shift;
-    my $parms = shift || {};
-    my $comment = shift;
-
-    # return from follow_link() is an HTTP::Response or undef
-    my $response = $self->follow_link( %$parms );
-
-    my $ok;
-    my $error;
-    if ( !$response ) {
-        $error = "No matching link found";
-    }
-    else {
-        if ( !$response->is_success ) {
-            $error = $response->as_string;
-        }
-        else {
-            $ok = 1;
-        }
-    }
-
-    $Test->ok( $ok, $comment );
-    $Test->diag( $error ) if $error;
-
-    return $ok;
-}
-
 =head2 $mech->stuff_inputs( [\%options] )
 
 Finds all free-text input fields (text, textarea, and password) in the
@@ -889,7 +1117,9 @@ sub stuff_inputs {
 
 =head1 TODO
 
-Add HTML::Lint and HTML::Tidy capabilities.
+Add HTML::Tidy capabilities.
+
+Add a broken image check.
 
 =head1 AUTHOR
 
@@ -898,9 +1128,8 @@ Andy Lester, C<< <andy at petdance.com> >>
 =head1 BUGS
 
 Please report any bugs or feature requests to
-C<bug-test-www-mechanize at rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Test-WWW-Mechanize>.
-I will be notified, and then you'll automatically be notified of progress on
+<http://code.google.com/p/www-mechanize/issues/list>.  I will be
+notified, and then you'll automatically be notified of progress on
 your bug as I make changes.
 
 =head1 SUPPORT
@@ -913,6 +1142,14 @@ You can also look for information at:
 
 =over 4
 
+=item * Google Code bug tracker
+
+L<http://code.google.com/p/www-mechanize/issues/list>
+
+Please B<do not use> the old queues for WWW::Mechanize and
+Test::WWW::Mechanize at
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Test-WWW-Mechanize>
+
 =item * AnnoCPAN: Annotated CPAN documentation
 
 L<http://annocpan.org/dist/Test-WWW-Mechanize>
@@ -921,10 +1158,6 @@ L<http://annocpan.org/dist/Test-WWW-Mechanize>
 
 L<http://cpanratings.perl.org/d/Test-WWW-Mechanize>
 
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Test-WWW-Mechanize>
-
 =item * Search CPAN
 
 L<http://search.cpan.org/dist/Test-WWW-Mechanize>
@@ -934,6 +1167,7 @@ L<http://search.cpan.org/dist/Test-WWW-Mechanize>
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to
+Greg Sheard,
 Michael Schwern,
 Mark Blackman,
 Mike O'Regan,
